#!/bin/bash

##########################################################################################
# This script loads ONAP Docker Container images rsynched from Kubernetes worker nodes,
# tags them using the original ONAP and branch tags to push to Nordix Harbor Container
# image registry. Once tagging is done, the images are pushed to Nordix registry.
#
# In order to enable traceability and reproducibility, the repo digests of the pushed
# images are extracted.
#
# As final step, the image metadata (repository, tag, and repo digest) is recorded in
# onap_docker_images.list file and this file gets stored in Nordix Artifact Repository
# as onap/online-install/<branch>/<promotion_id>/onap_docker_images.list and
# onap/online-install/<branch>/latest/onap_docker_images.list.
##########################################################################################

set -o errexit
set -o nounset
set -o pipefail

main() {

  # ensure credentials are set before proceeding with the rest
  if [[ -z ${NORDIX_ARM_TOKEN+x} ]] || [[ -z ${NORDIX_HARBOR_USERNAME+x} ]] || [[ -z ${NORDIX_HARBOR_PASSWORD+x} ]]; then
    echo "Error: One or more mandatory parameters are not set!"
    echo "       Ensure NORDIX_ARM_TOKEN, NORDIX_HARBOR_USERNAME, and NORDIX_HARBOR_PASSWORD are set before running this script!"
    exit 1
  fi

  # set defaults for optional variables if they are not set - they should be set by Jenkins job
  export ONAP_FILELIST="/tmp/onap_docker_images.list"
  export ONAP_VERSION=${ONAP_VERSION:-"master"}
  export NORDIX_ARM_REPO_URL=${NORDIX_ARM_REPO_URL:-"https://artifactory.nordix.org/artifactory"}
  export NORDIX_HARBOR_REPO_URL=${NORDIX_HARBOR_REPO_URL:-"registry.nordix.org"}
  export NORDIX_PROMOTION_ID=${NORDIX_PROMOTION_ID:-"$(date +%Y%m%d%H%M%S)"}

  # set ONAP repo urls
  export NORDIX_ONAP_ARM_REPO_URL="$NORDIX_ARM_REPO_URL/onap/online-install/${ONAP_VERSION}"
  export NORDIX_ONAP_HARBOR_REPO_URL="$NORDIX_HARBOR_REPO_URL/onap/online-install"

  # test repos
  #export NORDIX_ONAP_ARM_REPO_URL="$NORDIX_ARM_REPO_URL/onap/online-install/dummy"
  #export NORDIX_ONAP_HARBOR_REPO_URL="$NORDIX_HARBOR_REPO_URL/dummy"

  # login to registry.nordix.org
  docker login -u $NORDIX_HARBOR_USERNAME -p $NORDIX_HARBOR_PASSWORD $NORDIX_HARBOR_REPO_URL > /dev/null 2>&1

  # declare arrays to keep image metadata
  declare -A ONAP_IMAGE ONAP_IMAGES
  declare -A NORDIX_IMAGE NORDIX_IMAGES

  # ensure we don't have leftovers from previous execution
  sudo /bin/rm -f $ONAP_FILELIST

  # load images from tarfiles
  load_images

  # gather ONAP container image metadata from docker
  while read -r line; do
    ONAP_IMAGE_REPO=$(echo $line | cut -d'#' -f1)
    ONAP_IMAGE_TAG=$(echo $line | cut -d'#' -f2)

    # set Nordix image repo/tag
    NORDIX_IMAGE_REPO=$(echo $ONAP_IMAGE_REPO | sed "s|^nexus[^/]*|$NORDIX_ONAP_HARBOR_REPO_URL|g")
    NORDIX_IMAGE_TAG=$ONAP_IMAGE_TAG

    # tag and push image
    tag_push_image $ONAP_IMAGE_REPO $ONAP_IMAGE_TAG $NORDIX_IMAGE_REPO $NORDIX_IMAGE_TAG
  done < <(docker images --no-trunc --format '{{.Repository}}#{{.Tag}}' | grep 'nexus.*/onap' | sort)

  # generate onap filelist
  generate_onap_filelist $ONAP_FILELIST
}

load_images() {
  # get list of tarfiles
  cd /tmp/onap-container-images
  IMAGES_TO_LOAD=$(find . -type f -name '*.tar' | sort)

  # load images using docker load
  for IMAGE_TO_LOAD in $IMAGES_TO_LOAD; do
    docker load < $IMAGE_TO_LOAD #> /dev/null 2>&1
  done
}

tag_push_image() {
  ONAP_IMAGE_REPO=$1
  ONAP_IMAGE_TAG=$2
  NORDIX_IMAGE_REPO=$3
  NORDIX_IMAGE_TAG=$4

  # tag and push using Nordix tag
  docker tag $ONAP_IMAGE_REPO:$ONAP_IMAGE_TAG $NORDIX_IMAGE_REPO:$NORDIX_IMAGE_TAG
  docker push $NORDIX_IMAGE_REPO:$NORDIX_IMAGE_TAG > /dev/null 2>&1

  # tag and push using ONAP tag
  docker tag $ONAP_IMAGE_REPO:$ONAP_IMAGE_TAG $NORDIX_IMAGE_REPO:$ONAP_VERSION
  docker push $NORDIX_IMAGE_REPO:$ONAP_VERSION > /dev/null 2>&1

  echo "Tagged and pushed"
  echo "    $NORDIX_IMAGE_REPO:$NORDIX_IMAGE_TAG"
  echo "    $NORDIX_IMAGE_REPO:$ONAP_VERSION"
}

generate_onap_filelist() {
  ONAP_FILELIST=$1

  # add header to filelist to explain what this file is and the format
cat << EOF > $ONAP_FILELIST
##########################################################################################################################################
# This file contains list of ONAP Docker Container images and their metadata that are promoted from Nordix ONAP Online Install Pipelines #
# The lines are formatted as below                                                                                                       #
# <image repository>:<tag> #<repo_digest>                                                                                                #
# An example is                                                                                                                          #
# registry.nordix.org/dummy/onap/sdnc-ueb-listener-image:1.7.4 #sha256:29f63717044b985d296d1124cb61be7de26f8bf0c6399fda21c10d784e9757ef  #
##########################################################################################################################################
# Nordix Online Deployment Metadata
##########################################################################################################################################
# ___BEGIN___
# @NORDIX_PROMOTION_ID=$NORDIX_PROMOTION_ID                                                                                                            #
# ___END___
##########################################################################################################################################
EOF

  # gather ONAP container image metadata from docker
  while read -r line; do
    NORDIX_IMAGE_REPO=$(echo $line | cut -d'#' -f1)
    NORDIX_IMAGE_TAG=$(echo $line | cut -d'#' -f2)

    # get image repo_digest
    NORDIX_IMAGE_REPO_DIGEST=$(docker inspect $NORDIX_IMAGE_REPO:$NORDIX_IMAGE_TAG --format '{{index (.RepoDigests) 0}}' | cut -d '@' -f2)
    echo "$NORDIX_IMAGE_REPO:$NORDIX_IMAGE_TAG #$NORDIX_IMAGE_REPO_DIGEST" >> $ONAP_FILELIST

  done < <(docker images --no-trunc --format '{{.Repository}}#{{.Tag}}' | grep $NORDIX_HARBOR_REPO_URL | grep -v $ONAP_VERSION | sort)

  # calculate md5sum, shasum, sha256sum
  ONAP_FILELIST_MD5SUM=$(md5sum $ONAP_FILELIST | cut -d' ' -f1)
  ONAP_FILELIST_SHASUM=$(shasum $ONAP_FILELIST | cut -d' ' -f1)
  ONAP_FILELIST_SHA256SUM=$(sha256sum $ONAP_FILELIST | cut -d' ' -f1)

  # upload onap_docker_images.list to ARM
  echo "Stored ONAP Docker Images list as"
  for subfolder in $NORDIX_PROMOTION_ID latest; do
    curl -s \
        -H "X-JFrog-Art-Api:$NORDIX_ARM_TOKEN" \
        -H "X-Checksum-MD5:$ONAP_FILELIST_MD5SUM" \
        -H "X-Checksum-Sha1:$ONAP_FILELIST_SHASUM" \
        -H "X-Checksum-Sha256:$ONAP_FILELIST_SHA256SUM" \
        -T $ONAP_FILELIST \
        "$NORDIX_ONAP_ARM_REPO_URL/${subfolder}/onap_docker_images.list" > /dev/null 2>&1
    echo "    $NORDIX_ONAP_ARM_REPO_URL/${subfolder}/onap_docker_images.list"
  done
}

main "$@"
